#!/usr/bin/env python3

from typing import Any

from .tree import SourceNode, SourceIndex

"""Base class for extensions to the conversion process."""


class Extension:
    """The base class for all dragonglass extensions."""
    def __init__(self, config: dict[str, Any]):
        """
        Initialize the extension.

        Args:
            config (dict[str, Any]): Configuration values for this extension.
        """
        self._config = config

    @staticmethod
    def link_target(ctxt: Any, from_node: SourceNode, to_node: SourceNode) -> str:
        """
        Produces a link target value from one node to another.

        Args:
            ctxt (Context): Context for the operation.
            from_node (SourceNode): The node that is linking to another page.
            to_node (SourceNode): The node being linked to.

        Returns:
            str: The correct link target.
        """
        return to_node.link_target(ctxt.url_prefix, from_node if ctxt.relative_links else None)

    def loaded_metadata(self, ctxt: Any, node: SourceNode) -> None:
        """
        Called after the metadata for a node has been loaded.  Should be overridden by derived classes.

        Args:
            ctxt (Context): Context for the operation.
            node (SourceNode): Node that's just had its metadata loaded.
        """
        pass

    def indexed(self, ctxt: Any, index: SourceIndex) -> None:
        """
        Called after the index for the nodes has been generated.

        Args:
            ctxt (Context): Context for the operation.
            index (SourceIndex): The index to all nodes.
        """
        pass

    def supply_template_vars(self, ctxt: Any, node: SourceNode) -> dict[str, Any] | None:
        """
        Called to supply additional template variables to the template to be rendered.

        Args:
            ctxt (Context): Context for the operation.
            node (SourceNode): The node currently being processed for template rendering.

        Returns:
            dict[str, Any]: If not ``None``, contains additional template variables to be added before
                the template is rendered.
        """
        return None
